package CelBot::CommandContext;

use strict;

sub new
{
   my $class = shift;
   my %args = @_;

   my $self = bless \%args, $class;

   return $self;
}

sub new_sub
{
   my $parent = shift;
   my %args = @_;

   my $self = bless { %$parent }, ref $parent;

   # Remove the list_responder if %args provided a responder to replace it
   delete $self->{list_responder} if defined $args{responder};

   $self->{$_} = $args{$_} foreach keys %args;
   $self->{parent} = $parent;

   $self->{cascade} = 1 unless defined $self->{cascade};

   return $self;
}

sub scope
{
   my $self = shift;
   return $self->{scope};
}

sub cmduser
{
   my $self = shift;
   return $self->{cmduser};
}

sub allows_noise
{
   my $self = shift;
   return $self->{allows_noise} || 0;
}

sub text
{
   my $self = shift;
   return $self->{text};
}

sub respond
{
   my $self = shift;
   my ( @response ) = @_;

   my @lines;

   foreach my $r ( @response ) {
      # Protect against linefeeds in the line itself
      push @lines, split( m/\n/, $r );
   }

   if( defined $self->{max_spam} and @lines > $self->{max_spam} ) {
      # Truncate
      @lines = @lines[ 0 .. $self->{max_spam} - 1 ];
   }

   if( $self->{list_responder} ) {
      $self->{list_responder}->( @lines );
   }
   else {
      foreach my $line ( @lines ) {
         $self->{responder}->( $line );
      }
   }
}

sub respond_noise
{
   my $self = shift;
   $self->respond( @_ ) if $self->allows_noise;
}

sub allows_scope
{
   my $self = shift;
   my ( $allowedscope ) = @_;

   my $parent = $self->{parent};
   return $parent->allows_scope( $allowedscope ) if $parent and $self->{cascade}; # "Ask your mother"

   my $core = $self->{core};
   return $core->ask_plugins( "permit_command_scope", $allowedscope, $self );
}

sub allows_perm
{
   my $self = shift;
   my ( $allowedperm ) = @_;

   my $parent = $self->{parent};
   return $parent->allows_perm( $allowedperm ) if $parent and $self->{cascade}; # "Ask your mother"

   my $core = $self->{core};
   return $core->ask_plugins( "permit_command_perm", $allowedperm, $self );
}

sub allows_nonpublic
{
   my $self = shift;

   my $parent = $self->{parent};
   return $parent->allows_nonpublic if $parent and $self->{cascade}; # "Ask your mother"

   my $core = $self->{core};
   return $core->ask_plugins( "permit_command_nonpublic", $self );
}

# Now some convenient commandline parsing methods

sub peek_token
{
   my $self = shift;

   if( $self->{text} =~ m/^(\S+)\s/ ) {
      return $1;
   }
   elsif( $self->{text} =~ m/^(\S+)$/ ) {
      return $1;
   }
   else {
      return undef;
   }
}

sub pull_token
{
   my $self = shift;

   my $token = $self->peek_token;
   return undef unless defined $token;

   substr( $self->{text}, 0, length $token ) = "";
   $self->{text} =~ s/^\s+//; # Trim whitespace

   return $token;
}

# Keep perl happy; keep Britain tidy
1;
