package CelBot::Console;

use strict;

use base qw( Tickit::Console );

sub new
{
   my $class = shift;

   my $self = $class->SUPER::new(
      on_line => sub {
         my ( $console, $line ) = @_;

         if( $line =~ m{^/} ) {
            $console->do_internal_command( $line );
         }
         else {
            $console->active_tab->add_line( "<INPUT>: $line", indent => 9 );
         }
      },

      on_key => sub {
         my ( $console, $key ) = @_;

         if( $key =~ m/^M-(\d)/ ) {
            # Shortcut switch
            $console->activate_tab( $1 - 1 );
            return;
         }

         # Encode nicely
         $key =~ s/\//\\\\/g;
         $key =~ s/\n/\\n/g;
         $key =~ s/\r/\\r/g;
         $key =~ s/\e/\\e/g;
         $key =~ s{([^\x20-\x7e])}{sprintf "\\x%02x", ord $1}eg;
   
         $console->active_tab->add_line( "<KEY>: $key", indent => 7 );
      },
   );

   $self->{globaltab} = $self->add_tab(
      name => "GLOBAL",

      on_line => sub {
         my ( $console, $line ) = @_;

         my ( $cmd, $rest ) = (split m/ +/, $line, 2);

         if( my $func = $self->can( "command_$cmd" ) ) {
            $func->( $self, $console->active_tab, $rest );
         }
         else {
            $console->active_tab->add_line( "GLOBAL: Unrecognised global command $cmd", indent => 8 );
         }
      },
   );

   return $self;
}

sub do_internal_command
{
   my $self = shift;
   my ( $command ) = @_;

   if( $command =~ m{^/win (\d+)} ) {
      $self->switchto_tab( $1 - 1 );
   }
   else {
      $self->curtab->add_line( "Unrecognised command $command" );
   }
}

sub command_quit
{
   exit(0);
}

sub command_reload
{
   CelBot::Control::global_reload();
}

# Keep perl happy; keep Britain tidy
1;
