package CelBot::Plugin::Flood;

use strict;
use constant PLUGIN_TYPE => "flood";

sub new
{
   my $class = shift;
   my ( $core, $config ) = @_;

   my %channels;

   my $self = bless {
      core => $core,
      irc  => $core->get_plugin( "irc" ),
      channels => \%channels,
   }, $class;

   $config->associate_nodeset(
      'channel',
      '@name',

      add => sub {
         my ( $name, $config ) = @_;

         $channels{$name} = {},
         $self->reconfigure_channel( $channels{$name}, $config );
      },

      keep => sub {
         my ( $name, $config ) = @_;

         $self->reconfigure_channel( $channels{$name}, $config );
      },

      remove => sub {
         my ( $name ) = @_;

         delete $channels{$name};
      },
   );

   return $self;
}

sub reconfigure_channel
{
   my $self = shift;
   my ( $channelstruct, $config ) = @_;

   $channelstruct->{scores} = {};

   $channelstruct->{score_old} = $config->get_string( '@score_old', default => 5  );
   $channelstruct->{score_new} = $config->get_string( '@score_new', default => 10 );

   $channelstruct->{limit}     = $config->get_string( '@limit', default => 30 );
}

sub decay_scores
{
   my $self = shift;
   my ( $channelstruct, $now ) = @_;

   my $scores = $channelstruct->{scores};
   my $times  = $channelstruct->{times};

   foreach my $nick ( keys %$scores ) {
      my $newscore = $scores->{$nick} - ( $now - $times->{$nick} );
      if( $newscore > 0 ) {
         $scores->{$nick} = $newscore;
         $times->{$nick}  = $now;
      }
      else {
         delete $scores->{$nick};
         delete $times->{$nick};
      }
   }
}

sub on_channel_privmsg
{
   my $self = shift;
   my ( $channel, $user, $text ) = @_;

   my $channame = $channel->name;

   return 0 unless exists $self->{channels}->{$channame};
   my $channelstruct = $self->{channels}->{$channame};

   my $now = time();

   $self->decay_scores( $channelstruct, $now );

   my $joined_since = $now - ( $channel->get_jointime( $user ) || 0 );
   my $score_inc = $joined_since > 60 ? $channelstruct->{score_old} : $channelstruct->{score_new};

   my $nick = $user->nick_folded;

   my $score_now = ( $channelstruct->{scores}->{$nick} += $score_inc );
   $channelstruct->{times}->{$nick}  = $now;

   if( $score_now > $channelstruct->{limit} ) {
      $channel->kick( $user, "Flood (score: $score_now)" );
   }
}

sub on_user_rename
{
   my $self = shift;
   my ( $user, $oldnick, $newnick ) = @_;

   my $irc = $self->{irc};
   my $oldnick_folded = $irc->casefold_name( $oldnick );

   my $newnick_folded = $user->nick_folded;

   foreach my $channel ( keys %{ $self->{channels} } ) {
      my $scores = $self->{channels}->{$channel}->{scores};
      my $times  = $self->{channels}->{$channel}->{times};

      next unless exists $scores->{$oldnick_folded};

      $scores->{$newnick_folded} = delete $scores->{$oldnick_folded};
      $times->{$newnick_folded}  = delete $times->{$oldnick_folded};
   }
}

# Keep perl happy, keep Britain tidy
1;
