package CelBot::Plugin::NickServ;

use strict;
use constant PLUGIN_TYPE => "nickserv";

sub new
{
   my $class = shift;
   my ( $core, $config ) = @_;

   my $self = bless {
      core => $core,
   }, $class;

   return $self;
}

sub scatter_user_verify_handle
{
   my $self = shift;
   my ( $gencb, $user, $handle ) = @_;

   my $core = $self->{core};
   my $enabled = $core->ask_plugins( "ask_handle_data", $handle, "nickserv" );

   return unless $enabled;

   my $cb = $gencb->();

   $user->whois(
      on_whois => sub {
         my %whois = @_;

         if( $whois{307} ) {
            $core->log( "DEBUG", "<NickServ>", "WHOIS claims $user is recognised: " . join( " ", @{ $whois{307} } ) );
            $cb->(1);
         }
         else {
            $cb->(0);
         }
      },
   );
}

sub register_usermgmt_commands
{
   my $self = shift;
   my ( $usermgmt_plugin ) = @_;

   $usermgmt_plugin->register_command(
      plugin  => $self,
      command => "nickserv",
      perm    => 'self|master',

      subcmds => {
         get => { 
            args => [],
            summary => "Show current NickServ setting",
         },

         enable => { 
            args => [],
            summary => "Enable NickServ recognition",
         },

         disable => {
            args => [],
            summary => "Disable NickServ recognition",
         },
      },
      default => "get",

      summary => "Adjust use of NickServ for recognition",
   );
}

sub command_useracc_nickserv_get
{
   my $self = shift;
   my ( $context ) = @_;

   my $handle = $context->{handle};

   my $core = $self->{core};
   my $enabled = $core->ask_plugins( "ask_handle_data", $handle, "nickserv" );

   return "NickServ is " . ( $enabled ? "enabled" : "disabled" ) . " for $handle";
}

sub command_useracc_nickserv_enable
{
   my $self = shift;
   my ( $context ) = @_;

   my $handle = $context->{handle};

   my $core = $self->{core};
   $core->run_plugins( "do_handle_set_data", $handle, "nickserv", 1 );

   return "NickServ is now enabled for $handle";
}

sub command_useracc_nickserv_disable
{
   my $self = shift;
   my ( $context, $oldmask ) = @_;

   my $handle = $context->{handle};

   my $core = $self->{core};
   $core->run_plugins( "do_handle_set_data", $handle, "nickserv", 0 );

   return "NickServ is now disabled for $handle";
}

sub get_useracc_for_display
{
   my $self = shift;
   my ( $handle, $ret ) = @_;

   my $core = $self->{core};
   my $enabled = $core->ask_plugins( "ask_handle_data", $handle, "nickserv" );

   $ret->{nickserv} = "NickServ is " . ( $enabled ? "enabled" : "disabled" );

   return 1;
}

# Keep perl happy; keep Britain tidy
1;
