package CelBot::Plugin::PasswdAuth;

use strict;
use constant PLUGIN_TYPE => "passwdauth";

use Digest::MD5 qw( md5_base64 );

our $SALT = "57834902-y68923-nfy675923-n4-n49";

sub _hash_password
{
   my ( $pass ) = @_;
   return md5_base64( $SALT . $pass );
}

sub new
{
   my $class = shift;
   my ( $core, $config ) = @_;

   my $self = bless {
      core => $core,
   }, $class;

   return $self;
}

###
# Commands
###

sub register_usermgmt_commands
{
   my $self = shift;
   my ( $usermgmt_plugin ) = @_;

   $usermgmt_plugin->register_command(
      plugin  => $self,
      command => "password",
      perm    => 'self|master',

      subcmds => {
         set => {
            args => [ CelBot::Commands::ArgSpec::Bareword->new( 'pass' ) ],
            summary => "Set authentication password",
         },

         check => {
            args => [],
            summary => "Check if a password is set",
         },

         clear => {
            args => [],
            summary => "Clear authentication password",
         },
      },
      default => "check",

      summary => "User authentication by password",
   );
}

sub command_useracc_password_set
{
   my $self = shift;
   my ( $context, $pass ) = @_;

   my $handle = $context->{handle};

   my $core = $self->{core};
   $core->run_plugins( "do_handle_set_data", $handle, "passwdhash", _hash_password( $pass ) );

   return "Password now set";
}

sub command_useracc_password_check
{
   my $self = shift;
   my ( $context ) = @_;

   my $handle = $context->{handle};

   my $core = $self->{core};
   my $hash = $core->ask_plugins( "ask_handle_data", $handle, "passwdhash" );

   return defined $hash ? "Password set" : "No password";
}

sub command_useracc_password_clear
{
   my $self = shift;
   my ( $context ) = @_;

   my $handle = $context->{handle};

   my $core = $self->{core};
   $core->run_plugins( "do_handle_set_data", $handle, "passwdhash", undef );

   return "Password now cleared";
}

sub get_useracc_for_display
{
   my $self = shift;
   my ( $handle, $ret ) = @_;

   my $core = $self->{core};
   my $hash = $core->ask_plugins( "ask_handle_data", $handle, "passwdhash" );

   if( defined $hash ) {
      $ret->{passwdhash} = "Password is set";
   }
}

sub register_commands
{
   my $self = shift;
   my ( $commands_plugin ) = @_;

   $commands_plugin->register(
      plugin  => $self,
      command => "identify",
      perm    => 'public',

      args => [
         CelBot::Commands::ArgSpec::Bareword->new( 'handle' ),
         CelBot::Commands::ArgSpec::Bareword->new( 'pass' ),
      ],

      summary => "Identify as a registered user using a password",
   );
}

sub command_identify
{
   my $self = shift;
   my ( $context, $handle, $pass ) = @_;

   my $user = $context->cmduser;

   my $core = $self->{core};

   my $hash = $core->ask_plugins( "ask_handle_data", $handle, "passwdhash" );
   if( defined $hash and _hash_password( $pass ) eq $hash ) {
      $core->log( "AUDIT", "<PasswdAuth>", "$user successfully identified as $handle" );
      my $recognise_plugin = $core->get_plugin( "recognise" );
      $recognise_plugin->recognise( $user, $handle );
      return ( "Identify successful - you are now recognised as $handle" );
   }
   else {
      return ( "Identify failed - bad username or password" );
   }
}

# Keep perl happy; keep Britain tidy
1;
