package CelBot::Plugin::Say;

use strict;
use constant PLUGIN_TYPE => "say";

use CelBot::Commands;

sub new
{
   my $class = shift;
   my ( $core, $config ) = @_;

   my $self = bless {
      core => $core,
   }, $class;

   return $self;
}

sub register_commands
{
   my $self = shift;
   my ( $commands_plugin ) = @_;

   $commands_plugin->register(
      plugin  => $self,
      command => 'say',
      scope   => 'channel',
      perm    => "master",

      args => [
         CelBot::Commands::ArgSpec->new( 'message', eatall => 1 ),
      ],

      summary => "Send a PRIVMSG directly to a channel",
   );

   $commands_plugin->register(
      plugin  => $self,
      command => 'act',
      scope   => 'channel',
      perm    => "master",

      args => [
         CelBot::Commands::ArgSpec->new( 'action', eatall => 1 ),
      ],

      summary => "Send a CTCP ACTION directly to a channel",
   );
}

sub command_say
{
   my $self = shift;
   my ( $context, $message ) = @_;

   my $channel = $context->{channel};
   defined $channel or return ( "say: Need a channel" );

   $channel->privmsg( $message );
   return ();
}

sub command_act
{
   my $self = shift;
   my ( $context, $action ) = @_;

   my $channel = $context->{channel};
   defined $channel or return ( "act: Need a channel" );

   my $ctcp = "\001ACTION $action\001";

   $channel->privmsg( $ctcp );
   return ();
}

# Keep perl happy, keep Britain tidy
1;
