package CelBot::Plugin::UserDB::YAMLfile;

use strict;
use constant DB_TYPE => "yamlfile";
use constant PLUGIN_TYPE => undef;

use base qw( CelBot::Plugin::UserDB );

BEGIN {
   if( eval { require YAML::Syck } ) {
      YAML::Syck->import( qw( LoadFile DumpFile ) );
   }
   elsif( eval { require YAML } ) {
      YAML->import( qw( LoadFile DumpFile ) );
   }
   else {
      die "Can't find YAML::Syck or YAML";
   }
}

use Carp;

sub init
{
   my $self = shift;
   my ( $config ) = @_;

   my $filename = $config->get_string( '@file' );

   $self->{filename} = $filename;

   $self->{users}   = [];
   $self->{usermap} = {};

   $self->reload;
}

sub reload
{
   my $self = shift;

   return unless -e $self->{filename};

   my @users = LoadFile( $self->{filename} );

   undef @{ $self->{users} };
   undef %{ $self->{usermap} };

   foreach my $u ( @users ) {
      my %udata = %$u;

      push @{ $self->{users} }, \%udata;
      $self->{usermap}->{lc $udata{handle}} = \%udata;
   }
}

sub save
{
   my $self = shift;

   my @users;

   foreach my $u ( @{ $self->{users} } ) {
      my %udata = %$u;

      push @users, \%udata;
   }

   DumpFile( $self->{filename}, @users );
}

sub describe
{
   my $self = shift;
   return "YAML file $self->{filename}";
}

sub exists_handle
{
   my $self = shift;
   my ( $handle ) = @_;

   return exists $self->{usermap}->{lc $handle};
}

sub map_nick_to_handles
{
   my $self = shift;
   my ( $nick ) = @_;

   my @handles;

   foreach my $u ( @{ $self->{users} } ) {
      push @handles, $u->{handle} if grep { $_ eq $nick } @{ $u->{nicks} };
   }

   return @handles;
}

sub list_handles
{
   my $self = shift;
   # Can't use keys %usermap because they're casefolded
   return map { $_->{handle} } values %{ $self->{usermap} };
}

sub get_all_data
{
   my $self = shift;
   my ( $handle ) = @_;

   $handle = lc $handle;

   return undef unless exists $self->{usermap}->{$handle};

   my $data = $self->{usermap}->{$handle};

   return $data;
}

sub add_handle
{
   my $self = shift;
   my ( $handle ) = @_;

   $handle = lc $handle;

   croak "Cannot add handle $handle - already exists" if $self->exists_handle( $handle );

   my $data = {
      handle => $handle,
      global => {},
      channel => {},
   };

   push @{ $self->{users} }, $data;
   $self->{usermap}->{$handle} = $data;

   $self->mark_dirty;

   return $data;
}

# Keep perl happy; keep Britain tidy
1;
