package CelBot::RateLimiter;

use strict;

use Time::HiRes qw( time );

sub new
{
   my $class = shift;
   my ( $loop, $count, $delay ) = @_;

   my $self = bless {
      loop  => $loop,
      count => $count,
      delay => $delay,

      timequeue => [],
      deferred => [],
   }, $class;

   return $self;
}

sub do
{
   my $self = shift;
   my ( $code ) = @_;

   my $now = time;

   return if( $self->_try_immediate( $now, $code ) );

   my $deferred = $self->{deferred};

   $self->_install_timer() if !@$deferred;

   push @$deferred, $code;
}

sub _try_immediate
{
   my $self = shift;
   my ( $now, $code ) = @_;

   my $earlier = $now - $self->{delay};

   my $count = $self->{count};
   my $timequeue = $self->{timequeue};

   # Expire old times
   shift @$timequeue while @$timequeue and $timequeue->[0] < $earlier;

   if( @$timequeue < $count ) {
      $code->();

      push @$timequeue, $now;

      return 1;
   }
   else {
      return 0;
   }
}

sub _install_timer
{
   my $self = shift;

   my $until = $self->{timequeue}->[0] + $self->{delay};

   $self->{loop}->enqueue_timer(
      time => $until,
      code => sub { eval { $self->_run_deferrals; 1 } or 
                       ::console_print_err( "Deferred events failed - $@\n" ) },
   );
}

sub _run_deferrals
{
   my $self = shift;

   my $now = time;
   my $deferred = $self->{deferred};

   while( my $code = $deferred->[0] ) {
      $self->_try_immediate( $now, $code ) or last;

      shift @$deferred;
   }

   return if !@$deferred;

   $self->_install_timer();
}

# Keep perl happy, keep Britain tidy
1;
